﻿/* 
Win_hide.ahk

This module contains the functions that deal with the "Windows" menu that
is built into Quick Cliq. This menu deals with showing / hiding windows
of the user's choosing.

The Windows submenu has 2 built in items:
    Hide Current Window
        Selecting this will hide the window that currently has focus.

    Show All Windows
        This will "unhide" all of the windows that the user has hidden
        up to this point.


The "data/windows.ini" file:
    Quick Cliq uses a simple .ini file that is stored in the data directory
    caled "windows.ini". This file is only present when the user has hidden
    windows. When the first window is hidden, the file is created. When the
    last window is "unhidden", the file is destroyed.

    Here is a sample windows.ini file that has one hidden windows:

        [windows]
        0x80254=Administrator: C:\Windows\system32\cmd.exe - wmic

    Each line in the .ini file has the following format:
        WINDOW_HANDLE=WINDOW_TITLE

    So in the above .ini file, there is a window with a handle of 0x80254
    and a title of "Administrator: C:\Windows\system32\cmd.exe - wmic
*/

Build_Windows_Menu(Parent_Menu,MColor,TColor,IconsSize=16) {
	
	global
	local div, winIni, IniLine
	MMenu_Destroy(Windows_menu)
	Windows_menu := MMenu_Create("S" . IconsSize . " O" . ICON_OFFSET . " c" . MColor . " t" . TColor,global_LightMenu)
	
	DetectHiddenWindows, On
	winIni := FileOpen(WINDOWS_INI, "r`n")
	if IsObject(winIni)
		Loop
		{
			if winIni.AtEOF
				break
			If (A_Index = 1)
				Continue
			IniLine := winIni.ReadLine()
			if IniLine
			{
				Window_Handle := SubStr(IniLine, 1, inStr(IniLine,"=")-1)
				if !WinExist("ahk_id " . Window_Handle)
				{
					Remove_Window_From_Windows_Ini(Window_Handle)
					continue
				}
				Window_Title := SubStr(IniLine, inStr(IniLine,"=")+1)
				IconHandle := WinGetIcon( Window_Handle, global_Icons_Size )
				MMenu_Add(Windows_menu, Window_Title, iconhandle, "", "iWin_Hide_" . Window_Handle)
			}
		}
	winIni.Close()
	
	MMenu_Add(Windows_menu, "", "", "", "iWin_Hide_Sep s")	;Separator
	MMenu_Add(Windows_menu, "Hide Current Window", "", "", "iWin_Hide_Active_Window")
	MMenu_Add(Windows_menu, "Show All Windows", "", "", "iWin_Hide_Show_All")	;Add Show all windows entry in menu
	MMenu_Add(Windows_menu, "", "", "", "iWin_Hide_Sep2 s")
	MMenu_Add(Windows_menu, "Change Transparency", "", "", "iWin_Hide_ChangeTransp")
	MMenu_Add(Windows_menu, "Always-On-Top Toggle ", "", "", "iWin_Hide_ToggleOnTop")
	MMenu_Add(Windows_menu, "Close Button Toggle", "", "", "iWin_Hide_ToggleCloseBT")
	MMenu_Add(Windows_menu, "", "", "", "iWin_Hide_Sep3 s")
	MMenu_Add(Windows_menu, "Toggle Desktop", "", "", "iWin_Hide_ToggleDesktop")
	MMenu_Add(Windows_menu, "Tile Horizontally", "", "", "iWin_Hide_Tile_Horizontally")
	MMenu_Add(Windows_menu, "Tile Vertically", "", "", "iWin_Hide_Tile_Vertically")
	MMenu_Add(Windows_menu, "Cascade Windows", "", "", "iWin_Hide_Cascade")
	
	DetectHiddenWindows, Off
	If (Win_hide_ShowInMain = 1)
	{
		div := ColDiv(global_itemNum)
		MMenu_Add(Parent_Menu, "Windows", WINHIDE_ICO, "", "iWin_Hide_Menu m" . Windows_menu . div,global_IconsOnly)
		global_itemNum++
	}
	Window_Handle := Window_Title := IconHandle := ""
	Return
}


On_Hide_Window(window_id) {
	
	global
	
	If (Window_Get_Class(window_id) = "Shell_TrayWnd" || Window_Get_Class(window_id) = "Progman")
		Return
	
	Allow_hide_hotkey := 0
	WinGetTitle, window_title, ahk_id %window_id%
	IniWrite, %window_title%, %WINDOWS_INI%, windows, %Window_ID%
	
	items_count := MMenu_Count(Windows_menu)
	MMenu_Add(Windows_menu, window_title, WinGetIcon( Window_ID, global_Icons_Size ), items_count-11, "iWin_Hide_" . Window_ID) ;Add the chosen hidden window
	
	WH_fade := OptGet("WH_fade")
	if WH_fade not in 0,1
		WH_fade := DEFAULT_FADE
	if (WH_fade = 1)
	{
        Fade_Window_Out(Window_ID)
	}
	WinHide, ahk_id %Window_ID%
	DetectHiddenWindows, On
	WinSet, Transparent, 0, ahk_id %Window_ID%
	DetectHiddenWindows, Off
	Trans_num =
	window_title =
	WH_fade = 
	SendInput, !{escape}
	Allow_hide_hotkey := 1
	return
}

On_Show_Window(Window_Handle) {
    global
    Allow_hide_hotkey := 0
	selected_win_handle =
    DetectHiddenWIndows, On

    IfWinExist ahk_id %Window_Handle%
    {
        If !IsWindowVisible(Window_Handle)
        { 
            if (Get_Window_Fade_Setting() = 1) 
            { 
                Fade_Window_In(Window_Handle) 
            }
            else
            {
                Pop_Window_In(Window_Handle)
            }
        }
        Else ;if window preview active
            Pop_Window_In(Window_Handle)
    }
    Else
        Show_Missing_Window_Tooltip(Window_Handle)

    Remove_Window_From_Windows_Ini(Window_Handle)
    Remove_Window_From_Hidden_Windows_List(Window_Handle)

    DetectHiddenWindows, Off
    WH_fade =
    Allow_hide_hotkey := 1
    return
}


Pop_Window_In(Window_Handle)
{
    Show_Window(Window_Handle)
    Make_Window_Opaque(Window_Handle)
    Activate_Window(Window_Handle)
}


Show_Missing_Window_Tooltip(Window_Handle)
{
    IniRead, Read_Win, %WINDOWS_INI%, windows, %Window_Handle%
	DTP("Window was not found!`n""" . Read_Win . """`nIt appears that the parent process for this window does not exist anymore.",3000)
    Read_Win = 
}


/* Show_All_Hidden_Windows()
 *
 * This will show all of the windows that the user has hidden so far.
 */
Show_All_Hidden_Windows()
{
    global
    Loop, Read, %WINDOWS_INI%
    {
        If a_index = 1
        {
            Continue
        } 
		Else
        {
            StringSplit, Windows_Ini_Line_, A_LoopReadLine, =
            Window_Handle := Windows_Ini_Line_1
            Window_Title := Windows_Ini_Line_2
            Remove_Window_From_Hidden_Windows_List(Window_Handle)
            Show_Window(Window_Handle)
            Make_Window_Opaque(Window_Handle)
            Activate_Window(Window_Handle)
            Remove_Window_From_Windows_Ini(Window_Handle)
        }
    }

    ; This frees up the memory allocated by StringSplit array we created.
    Loop, %Windows_Ini_Line_0%
    {
        Windows_Ini_Line_%a_index% = 
    }
    DetectHiddenWindows, Off
    WH_fade =
    Allow_hide_hotkey := 1
    Return
}

Fade_Window_In(Window_Handle)
{
    Make_Window_Transparent(Window_Handle)
    Show_Window(WIndow_Handle)
    Activate_Window(Window_Handle)
    transparency = 0
    Loop, 10
    {
        transparency += 25.5
        Set_Window_Transparency(Window_Handle, transparency)
        sleep, 30
    }
}


Fade_Window_Out(Window_Handle)
{
    transparency = 255
    Loop, 10
    {
        transparency -= 25
        Set_Window_Transparency(Window_Handle, transparency)
        sleep, 30
    }	
}


DisableWin_Hide_Keys() 
{
	global
	WH_UseCSpace := OptGet("WH_UseCSpace")
	if WH_UseCSpace not in 0,1
		WH_UseCSpace := 1
	Hide_OptionalHotkey := OptGet("Hide_OptionalHotkey")
	if (Hide_OptionalHotkey == "ERROR" || Hide_OptionalHotkey == "")
		Hide_OptionalHotkey := HIDE_WINDOW_DEFAULT_HOTKEY
	Win_Hide_Show_Hotkey := OptGet("Win_Hide_Show_Hotkey")
	if (Win_Hide_Show_Hotkey == "ERROR" || Win_Hide_Show_Hotkey == "")
		Win_Hide_Show_Hotkey := WIN_HIDE_DEFAULT_HOTKEY
	Hotkey, ^Space, Win_Hotkey_Ex, Off UseErrorLevel
	if (WH_UseCSpace != 1)
		Hotkey(Hide_OptionalHotkey,0,"Win_Hotkey_Ex")
	Hotkey(Win_Hide_Show_Hotkey,0,"ShowWinHideMenu")
}